var acorn = require('acorn');
var _ = require('lodash');
var escodegen = require('escodegen');

function getExportsNode(nodes) {
  return _.find(nodes, function(node) {
    var exp = node.expression
    if (exp && exp.type === 'AssignmentExpression' && exp.left.type === 'MemberExpression' && exp.left.object.name === 'exports' && exp.right.type === 'ObjectExpression') {
      return exp.right;
    }
  })
}

function flat(property, parent, parentKey, isIdentifier) {
  var oldKey = property.key.value || property.key.name;
  var newKey;
  if (_.startsWith(oldKey, '&')) {
    newKey = oldKey.split('&').join(parentKey);
  } else {
    newKey = parentKey + ' ' + oldKey;
  }
  if (isIdentifier) {
    property.key.type = 'Literal';
  }
  property.key.value = newKey;
  parent.properties.push(property);
  return parent;
}

function flatten(object, parent, parentKey, stop) {
  _.each(object.properties, function(p) {
    var newParent = object;
    if ((p.key.type === 'Literal' || p.key.type === 'Identifier') && (p.value.type === 'ObjectExpression' || (p.value.type === 'CallExpression' && p.value.callee.property.name === '_extends')) && parent) {
      object.properties = _.without(object.properties, p);
      newParent = flat(p, parent, parentKey, p.key.type === 'Identifier');
    }

    if (p.value.type === 'ObjectExpression') {
      flatten(p.value, newParent, p.key.value || p.key.name, true);
    } else if (p.value.type === 'CallExpression') {
      _.each(p.value.arguments, function(a) {
        if (a.type === 'ObjectExpression') {
          flatten(a, newParent, p.key.value || p.key.name, true);
        }
      });
    }
  });
  object.properties = _.filter(object.properties, function(p) {
    if (p.value && p.value.properties && _.isEmpty(p.value.properties)) {
      return false;
    }
    return true;
  })
}

module.exports = function(content, map) {
  if (this.cacheable) {
    this.cacheable();
  }

  var tree = acorn.parse(content, {
    ecmaVersion: 6,
    sourceType: 'module'
  });
  var exportNode = getExportsNode(tree.body);
  flatten(exportNode.expression.right);
  return escodegen.generate(tree);
}
